@@ -1,5 +1,5 @@
 
-;; Copyright (C) 2008-2019, 2022 Tommi Höynälänmaa
+;; Copyright (C) 2008-2019, 2022, 2025 Tommi Höynälänmaa
 ;; Distributed under GNU Lesser General Public License version 3,
 ;; see file doc/LGPL-3.
 
@@ -74,7 +74,10 @@
             theme-with-exception-handler
             theme-apply-append
             load-module
-            module-var-exists?))
+            module-var-exists?
+            _vector-raw-contents
+            _vector-with-raw-contents0
+            _mutable-vector-with-raw-contents0))
 
 
 (eval-when (expand load eval)
@@ -86,6 +89,7 @@
 (use-modules (rnrs lists))
 (use-modules (rnrs io ports))
 (use-modules (srfi srfi-1))
+(use-modules (srfi srfi-43))
 (use-modules (ice-9 regex))
 (use-modules (oop goops))
 
@@ -93,6 +97,8 @@
 (use-modules (theme-d runtime theme-d-support))
 (use-modules (theme-d runtime runtime-theme-d-environment))
 (use-modules (theme-d runtime rte-base))
+(use-modules (theme-d runtime rte-object))
+(use-modules (theme-d runtime rte-vectors))
 (use-modules (theme-d runtime rte-globals))
 
 
@@ -708,3 +714,31 @@
 
 (define (module-var-exists? mod s-var-name)
   (if (module-variable mod s-var-name) #t #f))
+
+
+;; *** (standard-library raw-vector) ***
+
+
+;; Result type checking is handled by importing the procedure to Theme-D
+;; using prim-proc.
+(define (_vector-raw-contents v)
+  (slot-ref v 'v-field-values))
+
+
+(define (_vector-with-raw-contents0 tt-component rv)
+  (if (vector-every
+       (lambda (x) (is-instance? x tt-component))
+       rv)
+    (let ((cl (_i_make-param-class-inst1 _b_:vector (list tt-component))))
+      (make <rte-object> #:rcl-actual cl #:v-field-values rv))
+    (raise (make-theme-d-condition 'raw-vector-component-type-mismatch '()))))
+
+
+(define (_mutable-vector-with-raw-contents0 tt-component rv)
+  (if (vector-every
+       (lambda (x) (is-instance? x tt-component))
+       rv)
+    (let ((cl (_i_make-param-class-inst1 _b_:mutable-vector
+                                         (list tt-component))))
+      (make <rte-object> #:rcl-actual cl #:v-field-values rv))
+    (raise (make-theme-d-condition 'raw-vector-component-type-mismatch '()))))
